/*
 * Copyright (C) 2006-2008 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file audiod.h symbols exported from audiod.c */


int num_filters(int audio_format_num);
int get_audio_format_num(char *name);

/** enum of audio formats supported by para_audiod */
enum {AUDIOD_AUDIO_FORMATS_ENUM};

/** array of audio format names supported by para_audiod */
extern const char *audio_formats[];

/** maximal number of simultaneous instances */
#define MAX_STREAM_SLOTS 5

/**
 * the possible modes of operation
 *
 * - off: disconnect from para_server
 * - on: receive status information from para_server and play the audio stream
 * - sb: only receive status information but not the audio stream
 */
enum audiod_status_info {AUDIOD_OFF, AUDIOD_ON, AUDIOD_STANDBY};

/** defines one command of para_audiod */
struct audiod_command {
	/** the name of the command */
	const char *name;
	/** pointer to the function that handles the command */
	int (*handler)(int, int, char**);
	/**
	 * if the command prefers to handle the full line (rather than the usual
	 * argv[] array), it stores a pointer to the corresponding line handling
	 * function here. In this case, the above \a handler pointer must be NULL.
	 */
	int (*line_handler)(int, char*);
	/** one-line description of the command */
	const char *description;
	/** summary of the command line options */
	const char *usage;
	/** the long help text */
	const char *help;
};

/**
 * describes one instance of a receiver-filter-writer chain
 *
 * \sa receiver_node, receiver, filter, filter_node, filter_chain, writer,
 * writer_node, writer_node_group.
 */
struct slot_info {
	/** number of the audio format in this slot */
	int format;
	/** writer start time */
	struct timeval wstime;
	/** the receiver info associated with this slot */
	struct receiver_node *receiver_node;
	/** the active filter chain */
	struct filter_chain *fc;
	/** the active writer node group */
	struct writer_node_group *wng;
};

/**
 * the task for obtaining para_server's status (para_client stat)
 *
 * \sa struct task, struct sched
 */
struct status_task {
	/** the associated task structure of audiod */
	struct task task;
	/** client data associated with the stat task */
	struct client_task *ct;
	/** do not restart client command until this time */
	struct timeval restart_barrier;
	/** last time we received status data from para_server */
	struct timeval last_status_read;
	/** the offset value announced by para_server */
	int offset_seconds;
	/** the length of the current audio file as announced by para_server */
	int length_seconds;
	/** the start of the current stream from the view of para_server */
	struct timeval server_stream_start;
	/** the average time deviation between para_server and para_audiod */
	struct timeval sa_time_diff;
	/** whether client time is ahead of server time */
	int sa_time_diff_sign;
	/** non-zero if para_server's status is "playing" */
	int playing;
	/** number of times the clock difference is to be checked */
	unsigned clock_diff_count;
	/** when to start the next check for clock difference */
	struct timeval clock_diff_barrier;
	/** Number of the audio format as announced by para_server. */
	int current_audio_format_num;
};

extern struct slot_info slot[MAX_STREAM_SLOTS];
extern struct audiod_args_info conf;
extern int audiod_status;

void __noreturn clean_exit(int status, const char *msg);
int handle_connect(int accept_fd);
void audiod_status_dump(void);
void dump_empty_status(void);
char *get_time_string(struct timeval *newest_stime);

/** iterate over all slots */
#define FOR_EACH_SLOT(_slot) for (_slot = 0; _slot < MAX_STREAM_SLOTS; _slot++)
